#ifndef _FAT_H
#define _FAT_H

extern void init_fat( void);
extern void dir( unsigned long cluster);
extern void update_buf( unsigned char *buffer );

extern void read_lba(unsigned long ,unsigned char, unsigned char*);
extern unsigned long cluster_to_sector( unsigned long);
extern unsigned long get_next_cluster( unsigned long);

extern unsigned long PlaylistStartCluster;
extern unsigned long PlaylistLaenge;

extern unsigned char DirCount;
extern unsigned long last_cluster;
extern unsigned char DEVICE;
extern unsigned char SlavePresent;

#define MAGIC_OFFSET 		0x1FE
#define PAR_TABLE_OFFSET	0x1BE
//partition types
#define	FAT32				0x0b
#define	FAT32_LBA			0x0c
#define	FAT16_LBA			0x0e
#define	DOS16				0x04
#define DOS16_32			0x06

typedef char __s8;
typedef unsigned char __u8;
typedef unsigned short __u16;
typedef unsigned long __u32;

struct partition {
	unsigned char boot_ind;		/* 0x80 - active */
	unsigned char head;		/* starting head */
	unsigned char sector;		/* starting sector */
	unsigned char cyl;		/* starting cylinder */
	unsigned char sys_ind;		/* What partition type */
	unsigned char end_head;		/* end head */
	unsigned char end_sector;	/* end sector */
	unsigned char end_cyl;		/* end cylinder (0x08)*/
	unsigned long start_sect;	/* starting sector counting from 0 */
	unsigned long nr_sects;		/* nr of sectors in partition */
};

struct fat_boot_sector {
	__s8	ignored[3];	/* Boot strap short or near jump */
	__s8	system_id[8];	/* Name - can be used to special case
						   partition manager volumes */
	__u16	sector_size;	/* bytes per logical sector */
	__u8	cluster_size;	/* sectors/cluster */
	__u16	reserved;	/* reserved sectors */
	__u8	fats;		/* number of FATs (offset 0x10)*/
	__u16	dir_entries;	/*max! root directory entries */
	__u16	sectors;	/* number of sectors */
	__u8	media;		/* media code (unused) 22*/
	__u16	fat_length;	/* sectors/FAT */
	__u16	secs_track;	/* sectors per track */
	__u16	heads;		/* number of heads 28*/
	__u32	hidden;		/* hidden sectors (unused) 32*/
	__u32	total_sect;	/* number of sectors (if sectors == 0) 36*/

	/* The following fields are only used by FAT32 */
	__u32	fat32_length;	/* sectors/FAT */
	__u16	flags;		/* bit 8: fat mirroring, low 4: active fat */
	__u8	version[2];	/* major, minor filesystem version */
	__u32	root_cluster;	/* first cluster in root directory */
	__u16	info_sector;	/* filesystem info sector */
	__u16	backup_boot;	/* backup boot sector */
	__u16	reserved2[6];	/* Unused */
};


struct fat_boot_fsinfo {
	__u32   reserved1;	/* Nothing as far as I can tell */
	__u32   signature;	/* 0x61417272L */
	__u32   free_clusters;	/* Free cluster count.  -1 if unknown */
	__u32   next_cluster;	/* Most recently allocated cluster.
				 * Unused under Linux. */
	__u32   reserved2[4];
};

struct msdos_dir_entry {
	__u8	name[8],ext[3];	/* name and extension */
	__u8	attr;		/* attribute bits */
	__u8    lcase;		/* Case for base and extension */
	__u8	ctime_ms;	/* Creation time, milliseconds */
	__u16	ctime;		/* Creation time */
	__u16	cdate;		/* Creation date */
	__u16	adate;		/* Last access date */
	__u16   starthi;	/* High 16 bits of cluster in FAT32 (20)*/
	__u16	time,date;  /* time, date and first cluster */
	__u16	start;		/* Low 16 bits of cluster in FAT32 (26)*/
	__u32	size;		/* file size (in bytes) */
};
//size = 32byte

/* read-only */
#define ATTR_RO      1
/* hidden */
#define ATTR_HIDDEN  2
/* system */
#define ATTR_SYS     4
/* volume label */
#define ATTR_VOLUME  8
/* directory */
#define ATTR_DIR     16
/* archived */
#define ATTR_ARCH    32
/*ms_slot*/
#define ATTR_SLOT	0x0F

//flags beim slot aud 0x0F
/* Up to 13 characters of the name */
struct msdos_dir_slot {
	__u8    id;		/* sequence number for slot */
	__u8    name0_4[10];	/* first 5 characters in name */
	__u8    attr;		/* attribute byte */
	__u8    reserved;	/* always 0 */
	__u8    alias_checksum;	/* checksum for 8.3 alias */
	__u8    name5_10[12];	/* 6 more characters in name (14)*/
	__u16   start;		/* starting cluster number, 0 in long slots */
	__u8    name11_12[4];	/* last 2 characters in name */
};
#endif
